function load3

N = 100; %The number of discretization points in y
g = 9.81; %Gravitational constant
  
bucket.RearAngle = pi/3;
bucket.SideLength = 1.5; %m
bucket.Mass = 1e3; %kg
bucket.Volume = bucket.SideLength^2*sin(bucket.RearAngle)/2;
bucket.maxDTiltAngle = (pi/2-bucket.RearAngle/2)/N*5;

gravel.Density = 2e3; %kg/m^3
gravel.Angle = pi/4;
%For compression force:
gravel.Ktot = 6e4;
gravel.expCompr = 2;

yMax = 4; %The height the bucket should end at.

y = linspace(0, yMax, N)';
Y = y(2:(end-1));

% The starting point.
x0 = [Y/tan(gravel.Angle); bucket.RearAngle/2*ones(N-2,1)];
idx.numVar = length(x0); %Number of variables
idx.x = 1:length(Y); %Which variables are the x-variables
idx.bucketTiltAngle = length(Y) + (1:N-2); %Which variables are the
                                           %beta-variables

% Set up the auxiliary data.
options.auxdata = {y bucket g gravel idx};

% The constraint functions.
limits = constraintLimits(options.auxdata);
options.lb = limits.lb; %Lower bounds for the variables
options.ub = limits.ub; %Upper bounds for the variables
options.cl = limits.cl; %Lower bounds for the constraints
options.cu = limits.cu; %Upper bounds for the constraints

% Set the IPOPT options.  
options.ipopt.mu_strategy      = 'adaptive';
options.ipopt.max_iter         = 1000;
options.ipopt.tol              = 1e-8;
options.ipopt.print_level      = 0;

% The callback functions.
funcs.objective         = @objective;
funcs.constraints       = @constraints;
funcs.gradient          = @gradient;
funcs.jacobian          = @jacobian;
funcs.jacobianstructure = @jacobianstructure;
funcs.hessian           = @hessian;
funcs.hessianstructure  = @hessianstructure;

% Run IPOPT.
[xOpt info] = ipopt(x0, funcs, options);
info

objective(xOpt, options.auxdata)

bucket.x = [0; xOpt(idx.x); yMax/tan(gravel.Angle)];
bucket.y = y;
bucket.TiltAngle=[bucket.RearAngle/2; 
		  xOpt(idx.bucketTiltAngle); pi/2];
opts.makeMovie = false;
%opts..movieFileName = "Some file name to save the movie in"
drawLoader(bucket, gravel, opts)

% ------------------------------------------------------------------
function f = objective(xIn, auxdata)
%disp('obj')
[y bucket g gravel idx] = deal(auxdata{:});

x = xIn(idx.x);
bucketTiltAngle = xIn(idx.bucketTiltAngle);

dy = y(2)-y(1);
Y = y(2:end-1);
X = x-Y/tan(gravel.Angle);

%Potential energy:
f = -g* gravel.Density*sum(X.*Y)*dy;

%Compression energy:
tmp = [0; x; y(end)/tan(gravel.Angle)];
dx = (tmp(3:end)-tmp(1:end-2))/2;
K = gravel.Ktot;
n = gravel.expCompr;
f = f + K*sum(X.^n.*(cos(bucketTiltAngle-bucket.RearAngle/2).*dx + ...
		     sin(bucketTiltAngle-bucket.RearAngle/2)*dy));

% ------------------------------------------------------------------
function c = constraints(xIn, auxdata)
%disp('cons')
[y bucket g gravel idx] = deal(auxdata{:});

x = xIn(idx.x);
bucketTiltAngle = xIn(idx.bucketTiltAngle);

N = length(y);
s = bucket.SideLength;
dy = y(2)-y(1);
Y = y(2:end-1);
X = x - Y/tan(gravel.Angle);
gravelVolume = sum(X)*dy;

bta = [bucket.RearAngle/2; bucketTiltAngle; pi/2];

%Setting the constraints:
c = [gravelVolume - bucket.Volume;
     bta(2:end)-bta(1:end-1)
     Y-s*sin(bucketTiltAngle-bucket.RearAngle/2);
     x(2:end) - x(1:end-1)];

% ------------------------------------------------------------------
function limits = constraintLimits(auxdata)
%disp('consL')
[y bucket g gravel idx] = deal(auxdata{:});
N = length(y);
Y = y(2:end-1);

%Lower and upper bounds for decision variables
limits.lb = [Y/tan(gravel.Angle);
	     bucket.RearAngle/2*ones(length(idx.x),1)];
limits.ub = [y(end)/tan(gravel.Angle)*ones(length(idx.x),1);
	     pi/2*ones(length(idx.x),1)];

%Lower and upper bounds for constraints
limits.cl = [0;
	     -bucket.maxDTiltAngle*ones(N-1,1);
	     zeros(N-2,1);
	     zeros(N-3,1)];
limits.cu = [0;
	     bucket.maxDTiltAngle*ones(N-1,1);
	     inf*ones(N-2,1);
	     inf*ones(N-3,1)];

% ------------------------------------------------------------------
function grad = gradient(xIn, auxdata)
%disp('grad')
[y bucket g gravel idx] = deal(auxdata{:});

grad = zeros(idx.numVar, 1);
x = xIn(idx.x);
bucketTiltAngle = xIn(idx.bucketTiltAngle);

dy = y(2)-y(1);
Y = y(2:end-1);
%Potential energy gradient:
grad(idx.x) = -g*gravel.Density*dy* Y;

%Compression energy gradient:
X = x - Y/tan(gravel.Angle);
tmp = [0; x; y(end)/tan(gravel.Angle)];
dx = (tmp(3:end)-tmp(1:end-2))/2;
K = gravel.Ktot;
n = gravel.expCompr;
nXn1 = n*X.^(n-1);
t1 = K*nXn1.*(cos(bucketTiltAngle-bucket.RearAngle/2).*dx + ...
	      sin(bucketTiltAngle-bucket.RearAngle/2)*dy);
lx = length(idx.x);
t3 = sparse(ones(lx-1,1));
t2 = K*((diag(t3,1)-diag(t3,-1))'*...
	(X.^n.*cos(bucketTiltAngle-bucket.RearAngle/2))/2);
grad(idx.x) = grad(idx.x) + t1 + t2;
grad(idx.bucketTiltAngle) = ...
    K*(X.^n).*(-sin(bucketTiltAngle-bucket.RearAngle/2).*dx + ...
	       cos(bucketTiltAngle-bucket.RearAngle/2)*dy);

% ------------------------------------------------------------------
function J = jacobianstructure(auxdata)  
%disp('jacS')
[y bucket g gravel idx] = deal(auxdata{:});

N = length(y);
nx = length(idx.x);

tmp = [speye(nx); spalloc(1,nx,0)] + [spalloc(1,nx,0); speye(nx)];
J = [sparse(ones(1, nx)) spalloc(1, nx, 0);
     spalloc(nx+1,nx,0) tmp;
     spalloc(nx,nx,0) speye(nx);
     tmp(2:end-1,:) spalloc(N-3,nx,0)];

% ------------------------------------------------------------------
function J = jacobian(xIn, auxdata) 
%disp('jac')
[y bucket g gravel idx] = deal(auxdata{:});
N = length(y);
nx = length(idx.x);
dy = y(2)-y(1);
Y = y(2:end-1);

b = xIn(idx.bucketTiltAngle)-bucket.RearAngle/2;

tmp = [speye(nx); spalloc(1,nx,0)] - [spalloc(1,nx,0); speye(nx)];
J = [sparse(dy*ones(1, nx)) spalloc(1, nx, 0);
     spalloc(nx+1,nx,0) tmp;
     spalloc(nx,nx,0) diag(sparse(-bucket.SideLength*cos(b)));
     tmp(2:end-1,:) spalloc(N-3,nx,0)];

% ------------------------------------------------------------------
function H = hessianstructure (auxdata)
%disp('hessS')
[y bucket g gravel idx] = deal(auxdata{:});

nx = length(idx.x);
t = sparse(ones(nx,1));
if gravel.expCompr ~= 1,
  H11 = speye(nx) + diag(t(1:end-1),-1);
else
  H11 = diag(t(1:end-1),-1);
end
H12 = diag(t) + diag(t(1:end-1),1) + diag(t(1:end-1),-1);
H22 = diag(t);
H = [H11 spalloc(nx,nx,0); H12 H22];

% ------------------------------------------------------------------
function H = hessian(xIn, sigma, lambda, auxdata) 
%disp('hess')
[y bucket g gravel idx] = deal(auxdata{:});

x = xIn(idx.x);
b = xIn(idx.bucketTiltAngle) - bucket.RearAngle/2;

N = length(y);
dy = y(2)-y(1);
Y = y(2:end-1);
X = x - Y/tan(gravel.Angle);
tmp = [0; x; y(end)/tan(gravel.Angle)];
dx = (tmp(3:end)-tmp(1:end-2))/2;
n = gravel.expCompr;

%Hessian for objective function
t1 = n*(n-1)*diag(sparse(X.^(n-2).*(cos(b).*dx + sin(b)*dy)));
t2 = sparse(n*X.^(n-1).*cos(b)/2);
H11 = t1 + diag(t2(1:end-1)-t2(2:end), -1);

t1 = sparse(n*X.^(n-1).*(-sin(b).*dx + cos(b)*dy));
t2 = sparse(-X.^n.*sin(b)/2);
H12 = diag(t1) + diag(t2(1:end-1),1) + diag(-t2(2:end),-1);

H22 = diag(sparse(X.^n.*(-cos(b).*dx - sin(b)*dy)));
K = gravel.Ktot;
H = sigma*K*[H11 spalloc(length(x),length(b),0); H12 H22];

%Hessian for constraints function
ii = N+(1:N-2);
H = H + [0*H11 0*H12'; 
	 0*H12 diag(sparse(bucket.SideLength*sin(b).*lambda(ii)))];

% ------------------------------------------------------------------
%Only for debugging:
function g = numGrad(f, x)

h = 1e-6;
I = eye(length(x));
g = [];
for k = 1:length(x),
  f1 = f(x+h*I(:,k));
  f2 = f(x-h*I(:,k));
  g = [g (f1-f2)/2/h];
end

% ------------------------------------------------------------------
%Only for debugging:
function fi = funcIdx(f, x, idx)
fi = f(x);
fi = fi(idx, :)';